# Programmmer: Nate Page
# Date: August 08, 2011
# Filename: basics.py

# Basic functions for pygame.

import os.path
from pygame import *
import pygame

class Particle(pygame.sprite.Sprite):
    def __init__(self, pos, vx, vy, ax, ay, size, colorstructure, *groups):
        pygame.sprite.Sprite.__init__(self, groups)
        self.vx, self.vy, self.ax, self.ay = vx, vy, ax, ay
        self.images = []
        for x in colorstructure:
            start, end, duration = x
            startr, startg, startb = start
            endr, endg, endb = end
            def f(s, e, t):
                return s + int((e - s)*(t/float(duration)))
            for t in range(duration):
                image = pygame.Surface((size, size)).convert()
                image.fill((f(startr, endr, t), f(startg, endg, t), f(startb, endb, t)))
                self.images.append(image)
        self.image = self.images[0]
        self.rect = self.image.get_rect(center = pos)
    def update(self):
        self.rect.move_ip(self.vx, self.vy)
        self.vx = self.vx + self.ax
        self.vy = self.vy + self.ay
        if not self.images:
            self.kill()
        else:
            self.image = self.images.pop(0)

def rectFromImage(img, x, y):
    """ Creates a sprite rectangle from the given image at the given location. """
    rect = img.get_rect()  # Create a rectangle with the size of the image.
    rect = rect.move([x, y])  # Move the rectagle into position.
    return rect


def load_image(name, colorkey, x, y, game):
    """ Loads a image into memory. """
    fullname = os.path.join('data', game, name)  # Get the full path of the image.

    try:
        img = image.load(fullname)  # Load the image.
    except error, message:
        print 'Cannot load image:', fullname  # Print an error message.
        raise SystemExit, message  # Exit the game.
    
    img = img.convert()  # Convert the image into a proper format.
    
    if colorkey is not None:  # If a color key has been given...
        if colorkey is -1:  # If we have been told to autodetect the colorkey...
            colorkey = img.get_at((0,0))  # Automatically set the colorkey.
        img.set_colorkey(colorkey, RLEACCEL)  # Set the colorkey.
        
    return img, rectFromImage(img, x, y)


def load_sound(game, name):
    """ Loads a sound into memory. """
    class NoneSound:
        def play(self): pass
        
    if not mixer:  # If we cannot play sound...
        return NoneSound() # Return a useless sound.
    
    fullname = os.path.join('data', game, name)  # Get the full path to the image.
    
    try:
        sound = mixer.Sound(fullname)  # Load the sound.
    except error, message:
        print 'Cannot load sound:', name  # Print an error message.
        raise SystemExit, message  # Close the game.

    return sound

def draw_string(screen, string, color, x, y, size, fontname):
    """ Draws a string to the specified surface. """
    fnt = font.Font(fontname, size)  # Create a font.
    rndrd = fnt.render(string, True, color)  # Draw the string.
    screen.blit(rndrd, rectFromImage(rndrd, x, y))  # Draw the text to the screen.

def measure_string(string, fontname, size):
    """ Measures a string. """
    fnt = font.Font(fontname, size)  # Create a font.
    return fnt.size(string)
