from platformchar import *
import math, pygame, basics


class Camera():

    def __init__(self, sprite, size, overallsize):
        self.sprite = sprite
        self.x, self.y, self.sw, self.sh = sprite.rect.centerx, sprite.rect.centery, sprite.rect.w, sprite.rect.h
        self.w, self.h = size
        self.x -= self.w/2
        self.y -= self.h/2
        self.ow, self.oh = overallsize

    def changeSprite(self, position):
        self.x, self.y = position

    def resize(self, size):
        self.w, self.h = size

    def get_rect(self):

        self.x = self.sprite.rect.centerx - (self.w / 2)
        self.y = self.sprite.rect.centery - (self.h / 2)
        self.x = max(self.x, 0)
        self.y = max(self.y, 0)
        self.x = min(self.x, self.ow - self.w)
        self.y = min(self.y, self.oh - self.h)
        
        return pygame.Rect(self.x, self.y, self.w, self.h)


def run():

    pygame.init()
    pyScreen = pygame.display.set_mode((640, 480))
    pygame.display.set_caption('Tester')

    running = True

    level = basics.load_image("map2.bmp", None, 0, 0, "test")[0]

    screen = pygame.Surface((level.get_width()*16, level.get_height()*16))

    chars = pygame.sprite.Group()
    walls = pygame.sprite.Group()
    clock = pygame.time.Clock()

    levsize = level.get_height() * level.get_width() + 0.0
    
    for y in range(level.get_height()):
        for x in range(level.get_width()):
            if level.get_at((x, y))[0]:
                Wall(x*16, y*16, "tile.png", "test", None, walls)
                n = ((((level.get_width()*y) + x)/levsize)*100)
                pygame.event.pump()
                pygame.draw.rect(pyScreen, [128, 0, 0], pygame.Rect(270, 235, 100, 10))
                pygame.draw.rect(pyScreen, [0, 128, 0], pygame.Rect(270, 235, n, 10))
                pygame.display.flip()
                
    
    player = PlatformChar((24,64), "playa.png", "test", None, 135, walls, chars)
    cam = Camera(player, (640, 480), screen.get_size())

    while running:
        ev = pygame.event.get()

        for e in ev:
            if e.type == pygame.QUIT or (e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE):
                running = False

        rect = pygame.Rect(cam.get_rect())
        
        filt = rect
        filt.x -= 24
        filt.y -= 24
        filt.h += 48
        filt.w += 48
        def _filter(spr):
            return spr.rect.colliderect(filt)
        
        chars.update(ev)

        screen.fill([0,0,0])
        screen.set_clip(rect)
        
        chars.draw(screen)
        for wall in filter(_filter, walls):
            wall.draw(screen)

        pyScreen.fill([0,0,0])
        pyScreen.blit(screen, (0, 0), rect)
        basics.draw_string(pyScreen, str(clock.get_fps()), [0, 128, 0], 5, 5, 12, None)

        pygame.display.flip()

        clock.tick(60)

    pygame.quit()

if __name__ == "__main__":
    run()
