import math, pygame, basics

class PlatformChar(pygame.sprite.Sprite):

    def get_jump_height(self, speed):
        n = speed
        t = 0
        g = self.gravity
        h = 0
        ph = 0

        for x in range(5):  
            t += 1
            ph = h
            h += n - (t * g)

        while h >= ph:
            t += 1
            ph = h
            h += n - (t * g)

        return ph

    def set_jump_speed(self, goalheight):
        self.jumpspeed = 1
        while int(abs(self.get_jump_height(self.jumpspeed) - goalheight)) > 0:
            if self.get_jump_height(self.jumpspeed) < goalheight:
                self.jumpspeed += 0.001
            else:
                self.jumpspeed -= 0.001

        return self.jumpspeed

    
    def __init__(self, (x, y), img, game, colorkey, jumpheight, walls, *groups):

        # Sprite init.
        pygame.sprite.Sprite.__init__(self, groups)


        # Speed paramaters.
        self.hspeed = 0
        self.maxhspeed = 5
        self.vspeed = 0
        self.maxfallspeed = 20

        # Calculating jump speed.
        self.gravity = 0.5
        self.jumpspeed = -self.set_jump_speed(jumpheight)

        # Jumping and walls parameter.
        self.jumping = 0
        self.walls = walls

        # Rectangle and image.
        self.image, self.rect = basics.load_image(img, colorkey, x, y, game)

        # Keys
        self.keys = {'r':False, 'l':False}

        
    def accelerate_left(self):
        if self.hspeed > -self.maxhspeed:
            self.hspeed -= 1
            
    def slow_left(self):
        if self.hspeed < 0:
            self.hspeed += 1
            
    def accelerate_right(self):
        if self.hspeed < self.maxhspeed:
            self.hspeed += 1
            
    def slow_right(self):
        if self.hspeed > 0:
            self.hspeed -= 1
            
    def jump(self):
        self.vspeed = self.jumpspeed
        self.jumping = 1
        
    def stopjump(self):
        self.vspeed /= 2
        self.jumping = 0

    def update(self, events):

        for e in events:
            if e.type == pygame.KEYDOWN:
                if e.key == pygame.K_LEFT:
                    self.keys['l'] = True
                elif e.key == pygame.K_RIGHT:
                    self.keys['r'] = True
            elif e.type == pygame.KEYUP:
                if e.key == pygame.K_LEFT:
                    self.keys['l'] = False
                elif e.key == pygame.K_RIGHT:
                    self.keys['r'] = False

        if self.keys['l']:
            self.accelerate_left()
        else:
            self.slow_left()

        if self.keys['r']:
            self.accelerate_right()
        else:
            self.slow_right()
            
        self.vspeed += self.gravity

        if self.vspeed > self.maxfallspeed:
            self.vspeed = self.maxfallspeed

        if self.vspeed >= 0:
            self.jumping = 0

        # Only pay attention to nearby walls.
        filt = pygame.Rect(max(self.rect.x - self.rect.w, 0), max(self.rect.y - self.rect.h, 0), self.rect.x + 2*self.rect.w, self.rect.y + 2*self.rect.h)
        def _filter(spr):
            return spr.rect.colliderect(filt)
        
        for wall in filter(_filter, self.walls):
            if self.rect.x + self.rect.width > wall.x and self.rect.x < wall.x + wall.w:
                if self.vspeed > 0:
                    if self.rect.y + self.rect.height <= wall.y:
                        if self.rect.y + self.rect.height + self.vspeed > wall.y:
                            self.vspeed = wall.y - self.rect.y - self.rect.height

                if self.vspeed < 0:
                    if self.rect.y > wall.y:
                        if self.rect.y + self.vspeed < wall.y + wall.h:
                            self.vspeed = wall.y + wall.h - self.rect.y
                            
                if self.rect.y + self.rect.height == wall.y and pygame.key.get_pressed()[pygame.K_z]:
                    self.jump()
                

            if self.hspeed < 0:
                if self.rect.y + self.rect.height > wall.y and self.rect.y < wall.y + wall.h:
                    if self.rect.x > wall.x:
                        if self.rect.x + self.hspeed < wall.x + wall.w:
                            self.hspeed = wall.x + wall.w - self.rect.x
                            
            elif self.hspeed > 0:
                if self.rect.y + self.rect.height > wall.y and self.rect.y < wall.y + wall.h:
                    if self.rect.x < wall.x:
                        if self.rect.x + self.rect.width + self.hspeed > wall.x:
                            self.hspeed = wall.x - self.rect.x - self.rect.width

        self.rect.x += self.hspeed
        self.rect.y += self.vspeed
            
        if not pygame.key.get_pressed()[pygame.K_z] and self.jumping:
            self.stopjump()

            
class Wall(pygame.sprite.Sprite):
    def __init__(self, x, y, image, game, colorkey, *groups):
        
        pygame.sprite.Sprite.__init__(self, groups)
        self.image, self.rect = basics.load_image(image, colorkey, x, y, game)
        self.x = self.rect.x
        self.y = self.rect.y
        self.w = self.rect.w
        self.h = self.rect.h

    def draw(self, screen):
        screen.blit(self.image, self.rect)

def run():

    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    pygame.display.set_caption('Tester')

    running = True

    level = basics.load_image("map.bmp", None, 0, 0, "test")[0]

    chars = pygame.sprite.Group()
    walls = pygame.sprite.Group()
    clock = pygame.time.Clock()

    for y in range(30):
        for x in range(40):
            if level.get_at((x, y))[0]:
                Wall(x*16, y*16, "tile.png", "test", None, walls)
    
    player = PlatformChar((24,64), "playa.png", "test", None, 135, walls, chars)

    while running:
        ev = pygame.event.get()

        for e in ev:
            if e.type == pygame.QUIT or (e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE):
                running = False

        chars.update(ev)
        walls.update()

        screen.fill([0,0,0])
        
        chars.draw(screen)
        walls.draw(screen)

        pygame.display.flip()

        clock.tick(60)

    pygame.quit()


if __name__ == "__main__":
    run()
